;;; pacproxy.el --- Description -*- lexical-binding: t -*-

(require 'url)
(require 'url-proxy)
(require 'dbus)

(defvar pacproxy-domain-alist '(("SEI" . "ad.sei.cmu.edu"))
  "Mapping from network IDs to domain names.")

(defvar pacproxy--wpad-cache (make-hash-table :test #'equal))
(defvar pacproxy--original-locator nil)

(defun pacproxy--current-connection-name ()
  (let ((active (dbus-get-property
                  :system
                  "org.freedesktop.NetworkManager"
                  "/org/freedesktop/NetworkManager"
                  "org.freedesktop.NetworkManager"
                  "PrimaryConnection")))
    (dbus-get-property
                :system
                "org.freedesktop.NetworkManager"
                active
                "org.freedesktop.NetworkManager.Connection.Active"
                "Id")))

(defun pacproxy--retrieve-wpad (domain)
  (let ((result (gethash domain pacproxy--wpad-cache)))
    (if (eq result 'none) nil
      (or result
          (let* ((url-proxy-locator (lambda (&rest) "DIRECT"))
                 (pacurl (concat "http://wpad." domain "/wpad.dat"))
                 (result
                  (ignore-errors
                    (let ((result (url-retrieve-synchronously pacurl)))
                      (with-current-buffer result
                        (goto-char (point-min))
                        (re-search-forward "\n\n")
                        (delete-region (point-min) (point)))
                      (puthash domain result pacproxy--wpad-cache)
                      result))))
            (when (null result)
              (puthash domain 'none pacproxy--wpad-cache))
            result)))))

(defun pacproxy--url-destination (domain url host)
  (let ((wpad (pacproxy--retrieve-wpad domain)))
    (if (null wpad) "DIRECT"
      (with-temp-buffer
        (let ((result (current-buffer)))
          (with-current-buffer wpad
            (call-process-region
             (point-min) (point-max)
             "pactester" nil result nil
                 "-p" "-" "-u" url "-h" host))
          (buffer-string))))))

(defun pacproxy-find-proxy-for-url (urlobj host)
  (let ((manual (apply pacproxy--original-locator urlobj host nil)))
    (if (not (string-match "^DIRECT" manual))
        manual
      (let* ((found (assoc (pacproxy--current-connection-name)
                            pacproxy-domain-alist))
             (domain (when found (cdr found))))
        (if (null domain) "DIRECT"
          (pacproxy--url-destination
           domain (url-recreate-url urlobj) host))))))

(setq pacproxy--original-locator url-proxy-locator
      url-proxy-locator #'pacproxy-find-proxy-for-url)

(defun pacproxy-unload-function ()
  (setq url-proxy-locator pacproxy--original-locator)
  nil)

(provide 'pacproxy)
